"""Remove duplicated responses.

Reason:
https://github.com/RasaHQ/rasa-x/issues/1808

Revision ID: d5160f2d00b8
Revises: 8893bba2a522

"""
from alembic import op
import sqlalchemy as sa
from typing import Set, Tuple, Text

import rasax.community.database.schema_migrations.alembic.utils as utils

# revision identifiers, used by Alembic.
revision = "d5160f2d00b8"
down_revision = "8893bba2a522"
branch_labels = None
depends_on = None


def upgrade():
    remove_duplicated_templates()


def downgrade():
    pass


def remove_duplicated_templates() -> None:
    bind = op.get_bind()
    session = sa.orm.Session(bind=bind)

    templates_table = utils.get_reflected_table("template", session)
    templates = session.execute(templates_table.select()).fetchall()

    unique_template_tuples: Set[Tuple[Text, Text, Text]] = set()

    for t in templates:
        fingerprint = (t.project_id, t.template, t.text)
        if fingerprint in unique_template_tuples:
            delete_query = templates_table.delete().where(templates_table.c.id == t.id)
            session.execute(delete_query)
        else:
            unique_template_tuples.add(fingerprint)

    session.commit()
