import os
import uuid
import importlib.util
import rasax.community.config as rasa_x_config

from rasax.community.services.user_service import UserService, AuthMechanisms
from _pytest.capture import CaptureFixture

# we need to load the manage_users.py file from its path
manage_users_script_path = os.path.realpath(
    os.path.join(__file__, "..", "..", "..", "..", "..", "scripts", "manage_users.py")
)
module_spec = importlib.util.spec_from_file_location(
    "manage_users", manage_users_script_path
)
manage_users = importlib.util.module_from_spec(module_spec)
module_spec.loader.exec_module(manage_users)


def test_update_user_password(user_service: UserService):
    # create user
    username = "test999"
    password = "safe-pw123"
    role = "admin"

    user_service.create_user(
        username,
        password,
        rasa_x_config.team_name,
        role,
        AuthMechanisms.username_password,
    )

    # update password
    new_password = "better-pw123123"
    manage_users.change_password(user_service, username, new_password)

    # assert login works with new password
    assert user_service.login(username, new_password) is not None


def test_create_saml_user(user_service: UserService):
    name_id = "good-name-id"
    role = "admin"
    manage_users.create_saml_user(user_service, name_id, role, rasa_x_config.team_name)

    # assert SAML user exists and has expected role
    user = user_service.fetch_saml_user(name_id)
    assert user is not None
    assert user["roles"] == [role]


def test_list_users_script(user_service: UserService, capsys: CaptureFixture):
    username = uuid.uuid4().hex
    user_service.create_user(
        username,
        "pw",
        rasa_x_config.team_name,
        "admin",
        AuthMechanisms.username_password,
    )

    # run 'list' command and check output contains the new username
    manage_users.list_users(user_service, rasa_x_config.team_name)

    output = capsys.readouterr()
    assert username in output.out


def test_delete_user(user_service: UserService):
    # create user
    username = "user-to-delete01"
    user_service.create_user(
        username,
        "some-pw",
        rasa_x_config.team_name,
        "admin",
        AuthMechanisms.username_password,
    )

    # delete user and assert it's gone
    manage_users.delete_user(user_service, username)
    assert user_service.fetch_user(username) is None
